// Source file for Laplacian mesh class



#include <vector>
using namespace std;
#include "R3Shapes/R3Shapes.h"
#include "CSparse/CSparse.h"
#include "R3LaplacianMesh.h"



R3LaplacianMesh::
R3LaplacianMesh(void) 
  : R3Mesh()
{ 
  // Initialize all your variables to NULL here
  // PUT YOUR CODE HERE
}



R3LaplacianMesh::
~R3LaplacianMesh(void)
{
  // Delete all of your non-NULL variables here
  // PUT YOUR CODE HERE
}



int R3LaplacianMesh::
ReadMesh(const char *filename)
{
  // Read mesh -- do not change this line
  if (!R3Mesh::ReadFile(filename)) return 0;

  // Allocate Laplacian matrix and coordinates here
  // and initialize them based on the mesh topology and geometry
  // PUT YOUR CODE HERE

  // Return success
  return 1;
}



int R3LaplacianMesh::
WriteMesh(const char *filename)
{
  // Write mesh - do not change this line
  return R3Mesh::WriteFile(filename);
}



int R3LaplacianMesh::
ReadVertexValues(const char *filename)
{
  // Check number of vertices
  if (NVertices() == 0) {
    fprintf(stderr, "You must read the mesh before the function\n");
    return 0;
  }

  // Open file
  FILE *fp = fopen(filename, "r");
  if (!fp) {
    fprintf(stderr, "Unable to open function file: %s\n", filename);
    return 0;
  }

  // Read function values
  for (int i = 0; i < NVertices(); i++) {
    R3MeshVertex *vertex = Vertex(i);
    double value = 0;
    if (fscanf(fp, "%lf", &value) == (unsigned int) 1) {
      SetVertexValue(vertex, value);
    }
    else {
      fprintf(stderr, "Error reading value %d from %s\n", i, filename);
      return 0;
    }
  }

  // Close file
  fclose(fp);

  // Return success
  return 1;
}



int R3LaplacianMesh::
WriteVertexValues(const char *filename)
{
  // Open file
  FILE *fp = fopen(filename, "w");
  if (!fp) {
    fprintf(stderr, "Unable to open function file: %s\n", filename);
    return 0;
  }

  // Write function values
  for (int i = 0; i < NVertices(); i++) {
    R3MeshVertex *vertex = Vertex(i);
    fprintf(fp, "%g\n", VertexValue(vertex));
  }

  // Close file
  fclose(fp);

  // Return success
  return 1;
}



void R3LaplacianMesh::
UpdateVertexPositions(const vector<R3MeshVertex *>& anchored_vertices) 
{
  // Update all vertex positions except those in "anchored_vertices"
  // The position of the anchored_vertices will already have been set
  // prior to this function call.  You should solve for the positions
  // of the other vertices to satisfy the Laplacian coordinates in
  // the least squared sense and call SetVertexPosition(vertex, position)
  // to update the position in the R3Mesh.
  // PUT YOUR CODE HERE
}



void R3LaplacianMesh::
UpdateVertexValues(double empty_value)
{
  // Update the value of every vertex whose value == empty_value
  // (and leave the others unchanged) using interpolation based on the Laplacian.
  // PUT YOUR CODE HERE
}




void R3LaplacianMesh::
DrawVertexValues(double empty_value)
{
  // Compute statistics -- only first time 
  static double minimum_value = FLT_MAX;
  static double maximum_value = -FLT_MAX;
  if ((minimum_value == FLT_MAX) || (maximum_value == -FLT_MAX)) {
    for (int i = 0; i < NVertices(); i++) {
      R3MeshVertex *vertex = Vertex(i);
      double value = VertexValue(vertex);
      if (value < minimum_value) minimum_value = value;    
      if (value > maximum_value) maximum_value = value;    
    }
  }

  // Draw mesh shaded by vertex values
  if (minimum_value < maximum_value) {
    const RNRgb empty_color(1, 0.8, 0.2);
    for (int i = 0; i < NFaces(); i++) {
      R3MeshFace *face = Face(i);
      glBegin(GL_POLYGON);
      for (int j = 0; j < 3; j++) {
        RNRgb color(0,0,0);
        R3MeshVertex *vertex = VertexOnFace(face, j);
        double value = VertexValue(vertex);
        if (value == empty_value) {
          color = empty_color;
        }
        else {
          value = (value - minimum_value) / (maximum_value - minimum_value);
          if (value < 0.5) {
            color[0] = 1 - 2 * value;
            color[1] = 2 * value;
          }
          else {
            color[1] = 1 - 2 * (value - 0.5);
            color[2] = 2 * (value - 0.5);
          }
        }
        RNLoadRgb(color);
        R3LoadPoint(VertexPosition(vertex));
      }
      glEnd();
    }
  }
}
